package sun.reflect;

import sun.misc.VM;
import java.lang.reflect.Array;
import java.lang.reflect.Member;
import java.lang.reflect.Method;
import java.lang.reflect.Field;
import java.util.HashMap;
import java.lang.reflect.Modifier;
import java.util.Map;

public class Reflection
{
    private static volatile Map<Class<?>, String[]> fieldFilterMap;
    private static volatile Map<Class<?>, String[]> methodFilterMap;
    
    public Reflection() {
    }
    
    @CallerSensitive
    public static native Class<?> getCallerClass();
    
    @Deprecated
    public static native Class<?> getCallerClass(int p0);
    
    public static native int getClassAccessFlags(Class<?> p0);
    
    public static boolean quickCheckMemberAccess(Class<?> clazz, int n) {
        return Modifier.isPublic(getClassAccessFlags(clazz) & n);
    }
    
    public static void ensureMemberAccess(Class<?> clazz, Class<?> clazz2, Object o, int mod) throws IllegalAccessException {
        if (clazz == null || clazz2 == null) {
            throw new InternalError();
        }
        if (!verifyMemberAccess(clazz, clazz2, o, mod)) {
            throw new IllegalAccessException("Class " + clazz.getName() + " can not access a member of class " + clazz2.getName() + " with modifiers \"" + Modifier.toString(mod) + "\"");
        }
        return;
    }
    
    public static boolean verifyMemberAccess(Class<?> clazz, Class<?> clazz2, Object o, int n) {
        int n2 = 0;
        boolean n3 = false;
        if (clazz == clazz2) {
            return true;
        }
        if (!Modifier.isPublic(getClassAccessFlags(clazz2))) {
            n3 = isSameClassPackage(clazz, clazz2);
            n2 = 1;
            if (n3 == 0) {
                return false;
            }
        }
        if (Modifier.isPublic(n)) {
            return true;
        }
        int n4 = 0;
        if (Modifier.isProtected(n) && isSubclassOf(clazz, clazz2)) {
            n4 = 1;
        }
        if (n4 == 0 && !Modifier.isPrivate(n)) {
            if (n2 == 0) {
                n3 = isSameClassPackage(clazz, clazz2);
                n2 = 1;
            }
            if (!(n3 == 0)) {
                n4 = 1;
            }
        }
        if (n4 == 0) {
            return false;
        }
        if (Modifier.isProtected(n)) {
            Class clazz3 = (o == null) ? clazz2 : o.getClass();
            if (clazz3 != clazz) {
                if (n2 == 0) {
                    n3 = isSameClassPackage(clazz, clazz2);
                }
                if (n3 == 0 && !isSubclassOf(clazz3, clazz)) {
                    return false;
                }
            }
        }
        return true;
    }
    
    private static boolean isSameClassPackage(Class<?> clazz, Class<?> clazz2) {
        return isSameClassPackage(clazz.getClassLoader(), clazz.getName(), clazz2.getClassLoader(), clazz2.getName());
    }
    
    private static boolean isSameClassPackage(ClassLoader classLoader, String str, ClassLoader classLoader2, String s) {
        if (classLoader != classLoader2) {
            return false;
        }
        int lastIndex = str.lastIndexOf(46);
        int lastIndex2 = s.lastIndexOf(46);
        if (lastIndex == -1 || lastIndex2 == -1) {
            return lastIndex == lastIndex2;
        }
        int n = 0;
        int n2 = 0;
        if (str.charAt(n) == '[') {
            do {
                ++n;
            } while (str.charAt(n) == '[');
            if (str.charAt(n) != 'L') {
                throw new InternalError("Illegal class name " + str);
            }
        }
        if (s.charAt(n2) == '[') {
            do {
                ++n2;
            } while (s.charAt(n2) == '[');
            if (s.charAt(n2) != 'L') {
                throw new InternalError("Illegal class name " + s);
            }
        }
        int len = lastIndex - n;
        return len == lastIndex2 - n2 && str.regionMatches(false, n, s, n2, len);
    }
    
    static boolean isSubclassOf(Class<?> superclass, Class<?> clazz) {
        while (superclass != null) {
            if (superclass == clazz) {
                return true;
            }
            superclass = superclass.getSuperclass();
        }
        return false;
    }
    
    public static synchronized void registerFieldsToFilter(Class<?> clazz, String... array) {
        Reflection.fieldFilterMap = registerFilter(Reflection.fieldFilterMap, clazz, array);
    }
    
    public static synchronized void registerMethodsToFilter(Class<?> clazz, String... array) {
        Reflection.methodFilterMap = registerFilter(Reflection.methodFilterMap, clazz, array);
    }
    
    private static Map<Class<?>, String[]> registerFilter(Map<Class<?>, String[]> m, Class<?> obj, String... array) {
        if (m.get(obj) != null) {
            throw new IllegalArgumentException("Filter already registered: " + obj);
        }
        HashMap hashMap = new HashMap(m);
        hashMap.put(obj, array);
        return hashMap;
    }
    
    public static Field[] filterFields(Class<?> clazz, Field[] array) {
        if (Reflection.fieldFilterMap == null) {
            return array;
        }
        return (Field[])(Field[])filter(array, (String[])Reflection.fieldFilterMap.get(clazz));
    }
    
    public static Method[] filterMethods(Class<?> clazz, Method[] array) {
        if (Reflection.methodFilterMap == null) {
            return array;
        }
        return (Method[])(Method[])filter(array, (String[])Reflection.methodFilterMap.get(clazz));
    }
    
    private static Member[] filter(Member[] array, String[] array2) {
        if (array2 == null || array.length == 0) {
            return array;
        }
        int length = 0;
        for (Member member : array) {
            boolean n = false;
            int length3 = array2.length;
            int j = 0;
            while (j < length3) {
                if (member.getName() == array2[j]) {
                    n = true;
                    break;
                }
                ++j;
            }
            if (n == 0) {
                ++length;
            }
        }
        Member[] array3 = (Member[])(Member[])Array.newInstance(array[0].getClass(), length);
        int n2 = 0;
        for (Member member2 : array) {
            boolean n3 = false;
            int length5 = array2.length;
            int l = 0;
            while (l < length5) {
                if (member2.getName() == array2[l]) {
                    n3 = true;
                    break;
                }
                ++l;
            }
            if (n3 == 0) {
                array3[n2++] = member2;
            }
        }
        return array3;
    }
    
    public static boolean isCallerSensitive(Method method) {
        ClassLoader classLoader = method.getDeclaringClass().getClassLoader();
        return (VM.isSystemDomainLoader(classLoader) || isExtClassLoader(classLoader)) && method.isAnnotationPresent(CallerSensitive.class);
    }
    
    private static boolean isExtClassLoader(ClassLoader classLoader) {
        ClassLoader classLoader2 = ClassLoader.getSystemClassLoader();
        while (classLoader2 != null) {
            if (classLoader2.getParent() == null && classLoader2 == classLoader) {
                return true;
            }
            classLoader2 = classLoader2.getParent();
        }
        return false;
    }
    
    static {
        HashMap<Object, Object> fieldFilterMap = new HashMap();
        fieldFilterMap.put(Reflection.class, new String[] { "fieldFilterMap", "methodFilterMap" });
        fieldFilterMap.put(System.class, new String[] { "security" });
        fieldFilterMap.put(Class.class, new String[] { "classLoader" });
        Reflection.fieldFilterMap = fieldFilterMap;
        Reflection.methodFilterMap = new HashMap();
    }
}
